from setuptools import setup

setup(name='waterbear',
      description='A utility that makes it easy to use dot notation with python dictionaries',
      version='1.1.6',
      url='https://github.com/episodeyang/waterbear',
      author='Ge Yang',
      author_email='yangge1987@gmail.com',
      license=None,
      keywords=['waterbear', 'dict', 'dot-notation'],
      classifiers=[
          'Development Status :: 4 - Beta',
          'Intended Audience :: Science/Research',
          'Programming Language :: Python :: 3'
      ],
      packages=['waterbear'],
      install_requires=[]
      )
